#ifndef LTL_BASE_H
#define LTL_BASE_H

#include <setjmp.h>
#include <signal.h>
#include <stdlib.h>
#include <sys/types.h>
#include <ucontext.h>

#include <little/ltl_md.h>

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Define pointer to jmp buffer for supported platforms.
 */
#if __sgi
typedef __uint64_t      *ltl_jmp_buf;
#endif

#if __sparc || __alpha
typedef long		*ltl_jmp_buf;
#endif

#if __ppc__ || __hp9000s300
typedef int		*ltl_jmp_buf;
#endif

#if __hp9000s800
typedef double		*ltl_jmp_buf;
#endif

#if __linux__
typedef struct __jmp_buf_tag	*ltl_jmp_buf;
#endif

typedef unsigned short  ltl_short_t;
typedef int             ltl_key_t;

/*
 * Internal number type, used in
 */
typedef long            ltl_word_t;

/*
 * sigaltstack: stack growth direction.
 */
#if __sgi
# define SS_DOWN
# define REENTRANT_MALLOC
#endif

#ifdef LTL_THREADS
#include <pthread.h>
typedef pthread_t	ltl_thread_t;
typedef pthread_cond_t ltl_cond_t;
typedef pthread_mutex_t ltl_mutex_t;
#define LTL_MUTEX_INIT(mutex) {\
	pthread_mutexattr_t attr;\
	pthread_mutexattr_init(&attr);\
	pthread_mutexattr_settype(&attr,PTHREAD_MUTEX_RECURSIVE);\
	pthread_mutex_init((pthread_mutex_t *)(mutex),&attr);\
}
#define LTL_COND_INIT(cond) {\
	pthread_condattr_t attr;\
	pthread_condattr_init(&attr);\
	pthread_cond_init((ltl_cond_t *)(cond),&attr);\
}
#define LTL_COND_WAIT(cond,mutex) \
	pthread_cond_wait((ltl_cond_t *)(cond),(ltl_mutex_t *)(mutex))
#define LTL_COND_NOTIFY(cond) \
	pthread_cond_signal((ltl_cond_t *)(cond))
#define LTL_COND_NOTIFYALL(cond) \
	pthread_cond_broadcast((ltl_cond_t *)(cond))
#define LTL_COND_DESTROY(cond) \
	pthread_cond_destroy((ltl_cond_t *)(cond)) 
#define LTL_MUTEX_DESTROY(mutex) \
	pthread_mutex_destroy((pthread_mutex_t *)(mutex))
#define LTL_MUTEX_LOCK(mutex) \
	pthread_mutex_lock((pthread_mutex_t *)(mutex))
#define LTL_MUTEX_UNLOCK(mutex) \
	pthread_mutex_unlock((pthread_mutex_t *)(mutex))
#else
typedef int ltl_thread_t;
typedef int ltl_cond_t;
typedef int ltl_mutex_t;
#define LTL_COND_INIT(cond)
#define LTL_COND_WAIT(cond,mutex)
#define LTL_COND_NOTIFY(cond)
#define LTL_COND_NOTIFYALL(cond)
#define LTL_COND_DESTROY(cond)
#define LTL_MUTEX_INIT(mutex)
#define LTL_MUTEX_DESTROY(mutex)
#define LTL_MUTEX_LOCK(mutex)
#define LTL_MUTEX_UNLOCK(mutex)
#endif

void *ltl_malloc_g(size_t);
void *ltl_realloc_g(void *,size_t);
void ltl_free_g(void *);

#ifdef LTL_MEMDEBUG
#define ltl_malloc(size) ltl_malloc_g(size)
#define ltl_realloc(p,size) ltl_realloc_g((p),(size))
#define ltl_free(p) ltl_free_g(p)
#else
void *ltl_malloc(size_t);
void *ltl_realloc(void *,size_t);
void ltl_free(void *);
#endif

void ltl_malloc_sethandler(void (*)(size_t));

/*
 * Load a shared library into the library database.
 */
int ltl_load_library(const char *);

/*
 * Find a function handle in the library database.
 */
void *ltl_find_function(const char *);

/*
 * Unloads all DSOs.
 */
void ltl_unload_libs();

/*
 * Find current process' ucontext.
 */
int ltl_getcontext(ucontext_t *);

void ltl_blockall();
void ltl_unblockall();

#ifdef LTL_ASYNC
#define LTL_BLOCKALL() ltl_blockall()
#define LTL_UNBLOCKALL() ltl_unblockall()
#else
#define LTL_BLOCKALL()
#define LTL_UNBLOCKALL()
#endif

#ifdef __cplusplus
}
#endif

#endif
